'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var QNWebUploader = function (_EventEmitter) {
  (0, _inherits3.default)(QNWebUploader, _EventEmitter);

  function QNWebUploader(imgFile, uptoken, bsize, csize, url) {
    (0, _classCallCheck3.default)(this, QNWebUploader);

    var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

    _initialiseProps.call(_this);

    _this._imgFile = imgFile;
    _this._uptoken = uptoken;
    _this._qiniuUrl = url || QNWebUploader.QINIU_UPLOAD_URL;
    _this._bsize = bsize || QNWebUploader.BLOCK_SIZE;
    _this._csize = csize || QNWebUploader.CHUNK_SIZE;
    return _this;
  }

  QNWebUploader.prototype.cancel = function cancel() {
    this._cancel = true;
  };

  //失败后尝试的次数
  //## 上传片，也包含创建块
  // 参考:
  //  * http://developer.qiniu.com/docs/v6/api/reference/up/bput.html
  //  * http://developer.qiniu.com/docs/v6/api/reference/up/mkblk.html

  QNWebUploader.prototype._uploadChunk = function _uploadChunk(blob, lastPosition, ctx) {
    var _this2 = this;

    var url = void 0;
    var offset = lastPosition % this._bsize;
    var size = blob.size;
    if (offset === 0) {
      var blockSize = this._bsize;
      if (lastPosition + this._bsize > size) {
        blockSize = size - lastPosition;
      }
      url = this._qiniuUrl + 'mkblk/' + blockSize;
    } else {
      url = this._qiniuUrl + 'bput/' + ctx + '/' + offset;
    }
    var blobEnd = lastPosition + this._csize;
    blobEnd = blobEnd > size ? size : blobEnd;
    var chunk = blob.slice(lastPosition, blobEnd);

    return new _promise2.default(function (resolve, reject) {
      _this2._execPost(url, chunk, 'application/octet-stream', function (err, res) {
        if (err) {
          reject(err);
        } else {
          resolve({
            ctx: res.ctx,
            lastPosition: blobEnd,
            isBlockEnd: blobEnd % _this2._bsize === 0 || blobEnd === size
          });
        }
      });
    });
  };

  //## 创建文件
  // * http://developer.qiniu.com/docs/v6/api/reference/up/mkfile.html


  QNWebUploader.prototype._mkFile = function _mkFile(ctxList, size) {
    var _this3 = this;

    var url = this._qiniuUrl + 'mkfile/' + size + '/key/' + this._uptoken.key;

    return new _promise2.default(function (resolve, reject) {
      _this3._execPost(url, ctxList.join(','), 'text/plain', function (err, res) {
        if (err) {
          reject(err);
        } else {
          _this3._imgRes = res;
          resolve(res);
        }
      });
    });
  };

  QNWebUploader.prototype._execPost = function _execPost(url, data, contentType, cb) {
    var _this4 = this;

    var xhr = new XMLHttpRequest();
    var err = void 0;
    if (xhr.addEventListener) {
      xhr.addEventListener('load', function (e) {
        var res = JSON.parse(xhr.responseText);
        cb(err, res);
      });
      xhr.addEventListener('error', function (e) {
        if (_this4._errorTry) {
          _this4._errorTry--;
          _this4._execPost(url, data, contentType, cb);
        } else {
          cb(e, {});
        }
      });
    } else {
      xhr.onreadystatechange = function () {
        if (xhr.readyState == 4 && xhr.status == 200) {
          var res = JSON.parse(xhr.responseText);
          cb(err, res);
        }
      };
      xhr.onerror = function (e) {};
    }
    xhr.open('POST', url, true);
    xhr.setRequestHeader('Content-Type', contentType);
    xhr.setRequestHeader('Authorization', 'UpToken ' + this._uptoken.uptoken);
    xhr.send(data);
  };

  (0, _createClass3.default)(QNWebUploader, [{
    key: 'file',
    get: function get() {
      return this._imgFile;
    }
  }, {
    key: 'offset',
    get: function get() {
      return this._offset;
    }
  }, {
    key: 'percent',
    get: function get() {
      var offset = this._offset || 0;
      var size = this._imgFile.size || 1;
      return offset / size;
    }
  }, {
    key: 'imgRes',
    get: function get() {
      return this._imgRes;
    }
  }, {
    key: 'isCompleted',
    get: function get() {
      return !!this._imgRes;
    }

    //## 把生成预览和上传片结合起来

  }]);
  return QNWebUploader;
}(_events.EventEmitter);

QNWebUploader.QINIU_UPLOAD_URL = '//upload.qiniu.com/';
QNWebUploader.BLOCK_SIZE = 1024 * 1024 * 4;
QNWebUploader.CHUNK_SIZE = 1024 * 128;

var _initialiseProps = function _initialiseProps() {
  this.upload = function () {
    var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(queue) {
      var _this5 = this;

      var imgFile, position, currentCtx, ctxList, _ref, ctx, lastPosition, isBlockEnd, res;

      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              imgFile = this._imgFile;
              position = 0, currentCtx = '';
              ctxList = [];

            case 3:
              if (!(position < imgFile.size)) {
                _context.next = 18;
                break;
              }

              _context.next = 6;
              return this._uploadChunk(imgFile, position, currentCtx);

            case 6:
              _ref = _context.sent;
              ctx = _ref.ctx;
              lastPosition = _ref.lastPosition;
              isBlockEnd = _ref.isBlockEnd;

              if (!this._cancel) {
                _context.next = 12;
                break;
              }

              return _context.abrupt('return', new _promise2.default(function (resolve, reject) {
                _this5.emit('cancel');
                resolve({
                  msg: 'upload is cancel'
                });
              }));

            case 12:
              this._offset = position = lastPosition;
              this.emit('progress');
              currentCtx = ctx;
              if (isBlockEnd) {
                ctxList.push(ctx);
              }
              _context.next = 3;
              break;

            case 18:
              _context.next = 20;
              return this._mkFile(ctxList, imgFile.size);

            case 20:
              res = _context.sent;

              this.emit('complete', res);
              return _context.abrupt('return', new _promise2.default(function (resolve, reject) {
                resolve(res);
              }));

            case 23:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));
    return function (_x) {
      return ref.apply(this, arguments);
    };
  }();

  this._errorTry = 10;
};

exports.default = QNWebUploader;